"""The create_app() function"""

import chef
import flask

import sous_chef.blueprints

__all__ = ['create_app']


def configure_chef():
    chef_api_config = (
        flask.current_app.config['CHEF_URL'],
        flask.current_app.config['CHEF_KEY'],
        flask.current_app.config['CHEF_CLIENT'])

    if all(chef_api_config):
        flask.current_app.chef = chef.ChefAPI(*chef_api_config)
    else:
        flask.current_app.chef = chef.autoconfigure()

    flask.current_app.chef.set_default()
    flask.current_app.chef_environments = sorted(chef.Environment.list())


def create_app():
    app = flask.Flask('sous_chef')

    # Load configuration from defaults and an optional config file
    app.config.from_object('sous_chef.defaults')
    app.config.from_envvar('SOUS_CHEF_SETTINGS', silent=True)

    # Configure chef before the first request
    app.before_first_request(configure_chef)

    # Register blueprints - currently only the user interface
    app.register_blueprint(sous_chef.blueprints.ui)

    return app


def create_debug_app():
    app = create_app()
    app.config['DEBUG'] = True

    # Install the Flask debug toolbar extension, if the package is availible
    try:
        from flask_debugtoolbar import DebugToolbarExtension
    except ImportError:
        pass
    else:
        app.config['DEBUG_TB_INTERCEPT_REDIRECTS'] = False
        DebugToolbarExtension(app)

    return app



import setuptools

setuptools.setup(
    name="sous-chef",
    version="0.5.3",
    url="https://github.com/datasift/sous-chef",

    author="Sam Clements",
    author_email="sam.clements@datasift.com",

    description="A small webapp for viewing and searching Chef nodes",
    long_description=open('README.rst').read(),

    packages=setuptools.find_packages(),
    include_package_data=True,
    zip_safe=False,

    install_requires=[
        'flask==0.10.1',
        'pychef==0.2.3'
    ],

    extras_require={
        'debug': [
            'flask-debugtoolbar>=0.9.0'
        ],
        'deploy': [
            'gunicorn>=18.0'
        ]
    },

    entry_points={
        'console_scripts': [
            'sous-chef = sous_chef:main',
            'sous-chef-debug = sous_chef:debug'
        ]
    },

    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
)



"""Flask blueprints for sous-chef"""

import flask
import chef

__all__ = ['ui']

ui = flask.Blueprint('ui', __name__)


def default_environment():
    return flask.current_app.config.get('DEFAULT_CHEF_ENVIRONMENT', '_default')


def current_envionment():
    return flask.session.get('chef_environment', default_environment())


def search_nodes(**kwargs):
    kwargs.setdefault('chef_environment', current_envionment())
    search = ' AND '.join('{0}:{1}'.format(k, v) for k, v in kwargs.items())
    return chef.Search('node', search)


def search_node_names(**kwargs):
    return [row['name'] for row in search_nodes(**kwargs)]


class Node(chef.Node):
    @property
    def roles(self):
        return sorted(r[5:-1] for r in self.run_list if r.startswith('role'))


@ui.route('/', endpoint='home')
@ui.route('/roles/')
def roles():
    return flask.render_template('roles.html', roles=sorted(chef.Role.list()))


@ui.route('/roles/<string:name>')
def role(name):
    role = chef.Role(name)
    nodes = search_node_names(role=role.name)
    return flask.render_template('role.html', role=role, nodes=nodes)


@ui.route('/nodes/')
def nodes():
    return flask.render_template('nodes.html', nodes=search_node_names())


@ui.route('/nodes/<string:name>')
def node(name):
    return flask.render_template('node.html', node=Node(name))


@ui.before_request
def set_environment_variables():
    flask.g.chef_environments = flask.current_app.chef_environments
    flask.g.chef_environment = current_envionment()


# @ui.route('/environments/')
# def environments(name):
#     pass


# @ui.route('/environments/<string:name>')
# def environment(name):
#     pass


@ui.route('/environments/<string:name>/set')
def select_environment(name):
    if name in flask.g.chef_environments:
        flask.session['chef_environment'] = name
        flask.session.permanent = True
    return flask.redirect(flask.request.referrer or flask.url_for('ui.home'))
